import {
  scaleLinear,
  scaleOrdinal,
  extent,
  axisLeft,
  axisBottom,
  symbols,
  symbol,
} from 'd3';

//defining scatterplot function and its properties.

export const scatterPlot = () => {
  let xValue;
  let yValue;let width;
  let height;
  let data;
  let symbolValue;
  let margin;
  let size;

  const my = (selection) => {
    const x = scaleLinear()
      .domain(extent(data, xValue))
      .range([margin.left, width - margin.right]);

    const y = scaleLinear()
      .domain(extent(data, yValue))
      .range([height - margin.bottom, margin.top]);

    
    const symbolScale = scaleOrdinal() // defining symbols to be taken from symbols array.
      .domain(data.map(symbolValue))
      .range(symbols);

    const symbolGenerator = symbol().size(size);
    
    const marks = data.map((d) => ({
      x: x(xValue(d)),
      y: y(yValue(d)),
      pathD: symbolGenerator.type(symbolScale(symbolValue(d)))()
    }));

    selection
      .selectAll('path')
      .data(marks)
      .join('path')
      .attr('d', (d) => d.pathD)
      .attr('transform', (d) => `translate(${d.x},${d.y})`);

    selection
      .append('g')
      .attr('transform', `translate(${margin.left},0)`)
      .call(axisLeft(y));

    selection
      .append('g')
      .attr(
        'transform',
        `translate(0,${height - margin.bottom})`
      )
      .call(axisBottom(x));
  };

  //Defining various functions
  
  my.width = function (_) {
    return arguments.length ? ((width = +_), my) : width;
  };

  my.height = function (_) {
    return arguments.length ? ((height = +_), my) : height;
  };

  my.data = function (_) {
    return arguments.length ? ((data = _), my) : data;
  };

  my.xValue = function (_) {
    return arguments.length ? ((xValue = _), my) : xValue;
  };

  my.yValue = function (_) {
    return arguments.length ? ((yValue = _), my) : yValue;
  };

  my.symbolValue = function (_) {
    return arguments.length         // function to pick symbols
      ? ((symbolValue = _), my)
      : symbolValue;
  };

  my.margin = function (_) {
    return arguments.length ? ((margin = _), my) : margin; 
  };

  my.size = function (_) {
    return arguments.length ? ((size = +_), my) : size;  // for size of symbols
  };

  return my;
};
